import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='eml attachments ',
    from_='kaukinvk@yandex.ru',
    to=('imap.tools@ya.ru',),
    cc=(),
    bcc=(),
    reply_to=(),
    date=datetime.datetime(2019, 5, 1, 12, 20, 29, tzinfo=datetime.timezone(datetime.timedelta(seconds=18000))),
    date_str='Wed, 01 May 2019 12:20:29 +0500',
    text='UmVjZWl2ZWQ6IGZyb20gbXhmcm9udDJnLm1haWwueWFuZGV4Lm5ldCAoWzEyNy4wLjAuMV0pDQoJ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\r\nYjNmYmE0MGNjZDk2MDMwODRiZS0tDQo=UmVjZWl2ZWQ6IGZyb20gbXhmcm9udDJnLm1haWwueWFuZGV4Lm5ldCAoWzEyNy4wLjAuMV0pDQoJ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\r\nYjNmYmE0MGNjZDk2MDMwODRiZS0tDQo=',
    html='<div>two attach</div>',
    headers={'received': ('from mxback2g.mail.yandex.net ([127.0.0.1])\r\n\tby mxback2g.mail.yandex.net with LMTP id GCaAmtYs;\r\n\tWed, 1 May 2019 10:20:30 +0300', 'from mxback2g.mail.yandex.net (localhost.localdomain [127.0.0.1])\r\n\tby mxback2g.mail.yandex.net (Yandex) with ESMTP id C4C9426E11B7;\r\n\tWed,  1 May 2019 10:20:30 +0300 (MSK)', 'from localhost (localhost [::1])\r\n\tby mxback2g.mail.yandex.net (nwsmtp/Yandex) with ESMTP id HIJe2M7myP-KThSAlHL;\r\n\tWed, 01 May 2019 10:20:29 +0300', 'by myt5-262fb1897c00.qloud-c.yandex.net with HTTP;\r\n\tWed, 01 May 2019 10:20:29 +0300'), 'x-yandex-internal': ('1',), 'x-yandex-front': ('mxback2g.mail.yandex.net',), 'x-yandex-timemark': ('1556695229.976',), 'dkim-signature': ('v=1; a=rsa-sha256; c=relaxed/relaxed; d=yandex.ru; s=mail; t=1556695230;\r\n\tbh=Zmx9VIRnyRQRflnyRn1kpGYLdyO5YBC6/Wu9orXmTDk=;\r\n\th=Message-Id:Date:Subject:To:From;\r\n\tb=AM6LCew9Mz4XSIF78liQrYp4Kyg9RonJDizzaCNBRs5bOwla7bFphFQMIIYfmneJw\r\n\t 51WhDeo5L9ahRqG27no2kwmOggZ+Do99qY8oReCFObnfnaII6V2ZIvogKFXEfjHTB0\r\n\t bZlslZahe65zi1+xD7PnpeSWv8aYLZBgPuNJgA10=',), 'authentication-results': ('mxback2g.mail.yandex.net; dkim=pass header.i=@yandex.ru',), 'x-yandex-spam': ('1',), 'x-yandex-sender-uid': ('52494202',), 'from': ('=?utf-8?B?0JrQsNGD0LrQuNC9INCS0LvQsNC00LjQvNC40YA=?= <kaukinvk@yandex.ru>',), 'envelope-from': ('kaukinvk@yandex.ru',), 'to': ('imap.tools@ya.ru',), 'subject': ('eml attachments ',), 'mime-version': ('1.0',), 'x-mailer': ('Yamail [ http://yandex.ru ] 5.0',), 'date': ('Wed, 01 May 2019 12:20:29 +0500',), 'message-id': ('<8872861556695229@myt5-262fb1897c00.qloud-c.yandex.net>',), 'content-type': ('multipart/mixed;\r\n\tboundary="----==--bound.887287.myt5-262fb1897c00.qloud-c.yandex.net"',), 'return-path': ('kaukinvk@yandex.ru',), 'x-yandex-forward': ('d910e166fb8fe03380632bd988c8b67f',)},
    attachments=[
        dict(
            filename='yandex_email.eml',
            content_id='',
            content_disposition='attachment',
            content_type='message/rfc822',
            payload=b'Received: from mxfront2g.mail.yandex.net ([127.0.0.1])\r\n\tby mxfront2g.mail.yandex.net with LMTP id SONmnPb1\r\n\tfor <KaukinVK@yandex.ru>; Sun, 28 Apr 2019 14:32:37 +0300\r\nReceived: from out-3.smtp.github.com (out-3.smtp.github.com [192.30.252.194])\r\n\tby mxfront2g.mail.yandex.net (nwsmtp/Yandex) with ESMTPS id IxG07LRtjb-Warm4eXW;\r\n\tSun, 28 Apr 2019 14:32:36 +0300\r\n\t(using TLSv1.2 with cipher ECDHE-RSA-AES128-GCM-SHA256 (128/128 bits))\r\n\t(Client certificate not present)\r\nReturn-Path: noreply@github.com\r\nX-Yandex-Front: mxfront2g.mail.yandex.net\r\nX-Yandex-TimeMark: 1556451156.153\r\nAuthentication-Results: mxfront2g.mail.yandex.net; spf=pass (mxfront2g.mail.yandex.net: domain of github.com designates 192.30.252.194 as permitted sender, rule=[ip4:192.30.252.0/22]) smtp.mail=noreply@github.com; dkim=pass header.i=@github.com\r\nX-Yandex-Spam: 2\r\nX-Yandex-Fwd: OTM4OTM4Mjc2OTU3NTU3NTYyMyw5NzM5MTE2MTc5NzI0MTAxOTMw\r\nDate: Sun, 28 Apr 2019 04:32:35 -0700\r\nDKIM-Signature: v=1; a=rsa-sha256; c=relaxed/relaxed; d=github.com;\r\n\ts=pf2014; t=1556451155;\r\n\tbh=FsRPUKn3beupt2P3QB36UzfVESVClL07TVINnTX0nBg=;\r\n\th=Date:From:Reply-To:To:Cc:Subject:List-ID:List-Archive:List-Post:\r\n\t List-Unsubscribe:From;\r\n\tb=bzfQrv25o0IpZKiIrcGgLzDvM2VLfYficzjqSPPT0Rd3u3D66awwlxaVEHWqpE6dc\r\n\t 8ETR0+A3kN1V3xlo9Etlgn31105g3OQFG0jBMTdymdaXA/hu8Hrjr4Wf1krAGquHpq\r\n\t gJoF4ChuMuUhL554hguAmsWZ1BmzcjH6vVSa0ZEU=\r\nFrom: Buddy Rikard <notifications@github.com>\r\nReply-To: ikvk/imap_tools <reply+AAV6N2Z5ZMVYPVVMJBXEZE522LA5HEVBNHHBUHAPJU@reply.github.com>\r\nTo: ikvk/imap_tools <imap_tools@noreply.github.com>\r\nCc: Subscribed <subscribed@noreply.github.com>\r\nMessage-ID: <ikvk/imap_tools/issues/4@github.com>\r\nSubject: [ikvk/imap_tools] does not extract attached EML files (#4)\r\nMime-Version: 1.0\r\nContent-Type: multipart/alternative;\r\n boundary="--==_mimepart_5cc58f537542_49eb3fba40ccd9603084be";\r\n charset=UTF-8\r\nContent-Transfer-Encoding: 7bit\r\nPrecedence: list\r\nX-GitHub-Sender: TpyoKnig\r\nX-GitHub-Recipient: ikvk\r\nX-GitHub-Reason: subscribed\r\nList-ID: ikvk/imap_tools <imap_tools.ikvk.github.com>\r\nList-Archive: https://github.com/ikvk/imap_tools\r\nList-Post: <mailto:reply+AAV6N2Z5ZMVYPVVMJBXEZE522LA5HEVBNHHBUHAPJU@reply.github.com>\r\nList-Unsubscribe: <mailto:unsub+AAV6N2Z5ZMVYPVVMJBXEZE522LA5HEVBNHHBUHAPJU@reply.github.com>,\r\n <https://github.com/notifications/unsubscribe/AAV6N24RJYGGQBI5PKMGTK3PSWDNHANCNFSM4HI6JQUA>\r\nX-Auto-Response-Suppress: All\r\nX-GitHub-Recipient-Address: KaukinVK@yandex.ru\r\nX-Yandex-Forward: efb42d76edf7d5556112f3dac099406e\r\n\r\n\r\n----==_mimepart_5cc58f537542_49eb3fba40ccd9603084be\r\nContent-Type: text/plain;\r\n charset=UTF-8\r\nContent-Transfer-Encoding: 7bit\r\n\r\nAs .EML files have these headers:\r\n\r\n`--00000000000094809105878daac0\r\nContent-Type: message/rfc822; name="Totally legit email (2).eml"\r\nContent-Disposition: attachment; filename="Totally legit email (2).eml"\r\nContent-Transfer-Encoding: base64\r\nContent-ID: <f_jv0apprb0>\r\nX-Attachment-Id: f_jv0apprb0\r\n\r\nMIME-Version: 1.0\r\nDate: Fri, 25 Jan 2019 10:37:47 -0800\r\nMessage-ID: <CAKKXz3O5qFoTi68Ls4QCqZzQK8m6mASymTZ9ib6ysnBTOwwJOg@mail.gmail.com>\r\nSubject: Totally legit email\r\nFrom: REDACTED <REDACTED@DOMAIN.COM>\r\nTo: REDACTED <REDACTED@DOMAIN.COM>\r\nContent-Type: multipart/alternative; boundary="000000000000a30bb605804c9ff2"\r\n\r\n--000000000000a30bb605804c9ff2\r\nContent-Type: text/plain; charset="UTF-8"\r\nContent-Transfer-Encoding: quoted-printable`\r\n\r\nimap-tools does not return this as an attachment and completely disregards the attachment itself along with the filename. \r\n\r\n-- \r\nYou are receiving this because you are subscribed to this thread.\r\nReply to this email directly or view it on GitHub:\r\nhttps://github.com/ikvk/imap_tools/issues/4\r\n----==_mimepart_5cc58f537542_49eb3fba40ccd9603084be\r\nContent-Type: text/html;\r\n charset=UTF-8\r\nContent-Transfer-Encoding: 7bit\r\n\r\n<p>As .EML files have these headers:</p>\r\n<p>`--00000000000094809105878daac0<br>\r\nContent-Type: message/rfc822; name="Totally legit email (2).eml"<br>\r\nContent-Disposition: attachment; filename="Totally legit email (2).eml"<br>\r\nContent-Transfer-Encoding: base64<br>\r\nContent-ID: &lt;f_jv0apprb0&gt;<br>\r\nX-Attachment-Id: f_jv0apprb0</p>\r\n<p>MIME-Version: 1.0<br>\r\nDate: Fri, 25 Jan 2019 10:37:47 -0800<br>\r\nMessage-ID: <a href="mailto:CAKKXz3O5qFoTi68Ls4QCqZzQK8m6mASymTZ9ib6ysnBTOwwJOg@mail.gmail.com">CAKKXz3O5qFoTi68Ls4QCqZzQK8m6mASymTZ9ib6ysnBTOwwJOg@mail.gmail.com</a><br>\r\nSubject: Totally legit email<br>\r\nFrom: REDACTED <a href="mailto:REDACTED@DOMAIN.COM">REDACTED@DOMAIN.COM</a><br>\r\nTo: REDACTED <a href="mailto:REDACTED@DOMAIN.COM">REDACTED@DOMAIN.COM</a><br>\r\nContent-Type: multipart/alternative; boundary="000000000000a30bb605804c9ff2"</p>\r\n<p>--000000000000a30bb605804c9ff2<br>\r\nContent-Type: text/plain; charset="UTF-8"<br>\r\nContent-Transfer-Encoding: quoted-printable`</p>\r\n<p>imap-tools does not return this as an attachment and completely disregards the attachment itself along with the filename.</p>\r\n\r\n<p style="font-size:small;-webkit-text-size-adjust:none;color:#666;">&mdash;<br />You are receiving this because you are subscribed to this thread.<br />Reply to this email directly, <a href="https://github.com/ikvk/imap_tools/issues/4">view it on GitHub</a>, or <a href="https://github.com/notifications/unsubscribe-auth/AAV6N2ZD2P5MW33322GDAFTPSWDNHANCNFSM4HI6JQUA">mute the thread</a>.<img src="https://github.com/notifications/beacon/AAV6N27JEB4SBCE3VILPO6LPSWDNHANCNFSM4HI6JQUA.gif" height="1" width="1" alt="" /></p>\r\n<script type="application/json" data-scope="inboxmarkup">{"api_version":"1.0","publisher":{"api_key":"05dde50f1d1a384dd78767c55493e4bb","name":"GitHub"},"entity":{"external_key":"github/ikvk/imap_tools","title":"ikvk/imap_tools","subtitle":"GitHub repository","main_image_url":"https://github.githubassets.com/images/email/message_cards/header.png","avatar_image_url":"https://github.githubassets.com/images/email/message_cards/avatar.png","action":{"name":"Open in GitHub","url":"https://github.com/ikvk/imap_tools"}},"updates":{"snippets":[{"icon":"DESCRIPTION","message":"does not extract attached EML files (#4)"}],"action":{"name":"View Issue","url":"https://github.com/ikvk/imap_tools/issues/4"}}}</script>\r\n<script type="application/ld+json">[\r\n{\r\n"@context": "http://schema.org",\r\n"@type": "EmailMessage",\r\n"potentialAction": {\r\n"@type": "ViewAction",\r\n"target": "https://github.com/ikvk/imap_tools/issues/4",\r\n"url": "https://github.com/ikvk/imap_tools/issues/4",\r\n"name": "View Issue"\r\n},\r\n"description": "View this Issue on GitHub",\r\n"publisher": {\r\n"@type": "Organization",\r\n"name": "GitHub",\r\n"url": "https://github.com"\r\n}\r\n}\r\n]</script>\r\n----==_mimepart_5cc58f537542_49eb3fba40ccd9603084be--\r\n',
        ),
        
        dict(
            filename='yandex_email.eml',
            content_id='',
            content_disposition='attachment',
            content_type='message/rfc822',
            payload=b'Received: from mxfront2g.mail.yandex.net ([127.0.0.1])\r\n\tby mxfront2g.mail.yandex.net with LMTP id SONmnPb1\r\n\tfor <KaukinVK@yandex.ru>; Sun, 28 Apr 2019 14:32:37 +0300\r\nReceived: from out-3.smtp.github.com (out-3.smtp.github.com [192.30.252.194])\r\n\tby mxfront2g.mail.yandex.net (nwsmtp/Yandex) with ESMTPS id IxG07LRtjb-Warm4eXW;\r\n\tSun, 28 Apr 2019 14:32:36 +0300\r\n\t(using TLSv1.2 with cipher ECDHE-RSA-AES128-GCM-SHA256 (128/128 bits))\r\n\t(Client certificate not present)\r\nReturn-Path: noreply@github.com\r\nX-Yandex-Front: mxfront2g.mail.yandex.net\r\nX-Yandex-TimeMark: 1556451156.153\r\nAuthentication-Results: mxfront2g.mail.yandex.net; spf=pass (mxfront2g.mail.yandex.net: domain of github.com designates 192.30.252.194 as permitted sender, rule=[ip4:192.30.252.0/22]) smtp.mail=noreply@github.com; dkim=pass header.i=@github.com\r\nX-Yandex-Spam: 2\r\nX-Yandex-Fwd: OTM4OTM4Mjc2OTU3NTU3NTYyMyw5NzM5MTE2MTc5NzI0MTAxOTMw\r\nDate: Sun, 28 Apr 2019 04:32:35 -0700\r\nDKIM-Signature: v=1; a=rsa-sha256; c=relaxed/relaxed; d=github.com;\r\n\ts=pf2014; t=1556451155;\r\n\tbh=FsRPUKn3beupt2P3QB36UzfVESVClL07TVINnTX0nBg=;\r\n\th=Date:From:Reply-To:To:Cc:Subject:List-ID:List-Archive:List-Post:\r\n\t List-Unsubscribe:From;\r\n\tb=bzfQrv25o0IpZKiIrcGgLzDvM2VLfYficzjqSPPT0Rd3u3D66awwlxaVEHWqpE6dc\r\n\t 8ETR0+A3kN1V3xlo9Etlgn31105g3OQFG0jBMTdymdaXA/hu8Hrjr4Wf1krAGquHpq\r\n\t gJoF4ChuMuUhL554hguAmsWZ1BmzcjH6vVSa0ZEU=\r\nFrom: Buddy Rikard <notifications@github.com>\r\nReply-To: ikvk/imap_tools <reply+AAV6N2Z5ZMVYPVVMJBXEZE522LA5HEVBNHHBUHAPJU@reply.github.com>\r\nTo: ikvk/imap_tools <imap_tools@noreply.github.com>\r\nCc: Subscribed <subscribed@noreply.github.com>\r\nMessage-ID: <ikvk/imap_tools/issues/4@github.com>\r\nSubject: [ikvk/imap_tools] does not extract attached EML files (#4)\r\nMime-Version: 1.0\r\nContent-Type: multipart/alternative;\r\n boundary="--==_mimepart_5cc58f537542_49eb3fba40ccd9603084be";\r\n charset=UTF-8\r\nContent-Transfer-Encoding: 7bit\r\nPrecedence: list\r\nX-GitHub-Sender: TpyoKnig\r\nX-GitHub-Recipient: ikvk\r\nX-GitHub-Reason: subscribed\r\nList-ID: ikvk/imap_tools <imap_tools.ikvk.github.com>\r\nList-Archive: https://github.com/ikvk/imap_tools\r\nList-Post: <mailto:reply+AAV6N2Z5ZMVYPVVMJBXEZE522LA5HEVBNHHBUHAPJU@reply.github.com>\r\nList-Unsubscribe: <mailto:unsub+AAV6N2Z5ZMVYPVVMJBXEZE522LA5HEVBNHHBUHAPJU@reply.github.com>,\r\n <https://github.com/notifications/unsubscribe/AAV6N24RJYGGQBI5PKMGTK3PSWDNHANCNFSM4HI6JQUA>\r\nX-Auto-Response-Suppress: All\r\nX-GitHub-Recipient-Address: KaukinVK@yandex.ru\r\nX-Yandex-Forward: efb42d76edf7d5556112f3dac099406e\r\n\r\n\r\n----==_mimepart_5cc58f537542_49eb3fba40ccd9603084be\r\nContent-Type: text/plain;\r\n charset=UTF-8\r\nContent-Transfer-Encoding: 7bit\r\n\r\nAs .EML files have these headers:\r\n\r\n`--00000000000094809105878daac0\r\nContent-Type: message/rfc822; name="Totally legit email (2).eml"\r\nContent-Disposition: attachment; filename="Totally legit email (2).eml"\r\nContent-Transfer-Encoding: base64\r\nContent-ID: <f_jv0apprb0>\r\nX-Attachment-Id: f_jv0apprb0\r\n\r\nMIME-Version: 1.0\r\nDate: Fri, 25 Jan 2019 10:37:47 -0800\r\nMessage-ID: <CAKKXz3O5qFoTi68Ls4QCqZzQK8m6mASymTZ9ib6ysnBTOwwJOg@mail.gmail.com>\r\nSubject: Totally legit email\r\nFrom: REDACTED <REDACTED@DOMAIN.COM>\r\nTo: REDACTED <REDACTED@DOMAIN.COM>\r\nContent-Type: multipart/alternative; boundary="000000000000a30bb605804c9ff2"\r\n\r\n--000000000000a30bb605804c9ff2\r\nContent-Type: text/plain; charset="UTF-8"\r\nContent-Transfer-Encoding: quoted-printable`\r\n\r\nimap-tools does not return this as an attachment and completely disregards the attachment itself along with the filename. \r\n\r\n-- \r\nYou are receiving this because you are subscribed to this thread.\r\nReply to this email directly or view it on GitHub:\r\nhttps://github.com/ikvk/imap_tools/issues/4\r\n----==_mimepart_5cc58f537542_49eb3fba40ccd9603084be\r\nContent-Type: text/html;\r\n charset=UTF-8\r\nContent-Transfer-Encoding: 7bit\r\n\r\n<p>As .EML files have these headers:</p>\r\n<p>`--00000000000094809105878daac0<br>\r\nContent-Type: message/rfc822; name="Totally legit email (2).eml"<br>\r\nContent-Disposition: attachment; filename="Totally legit email (2).eml"<br>\r\nContent-Transfer-Encoding: base64<br>\r\nContent-ID: &lt;f_jv0apprb0&gt;<br>\r\nX-Attachment-Id: f_jv0apprb0</p>\r\n<p>MIME-Version: 1.0<br>\r\nDate: Fri, 25 Jan 2019 10:37:47 -0800<br>\r\nMessage-ID: <a href="mailto:CAKKXz3O5qFoTi68Ls4QCqZzQK8m6mASymTZ9ib6ysnBTOwwJOg@mail.gmail.com">CAKKXz3O5qFoTi68Ls4QCqZzQK8m6mASymTZ9ib6ysnBTOwwJOg@mail.gmail.com</a><br>\r\nSubject: Totally legit email<br>\r\nFrom: REDACTED <a href="mailto:REDACTED@DOMAIN.COM">REDACTED@DOMAIN.COM</a><br>\r\nTo: REDACTED <a href="mailto:REDACTED@DOMAIN.COM">REDACTED@DOMAIN.COM</a><br>\r\nContent-Type: multipart/alternative; boundary="000000000000a30bb605804c9ff2"</p>\r\n<p>--000000000000a30bb605804c9ff2<br>\r\nContent-Type: text/plain; charset="UTF-8"<br>\r\nContent-Transfer-Encoding: quoted-printable`</p>\r\n<p>imap-tools does not return this as an attachment and completely disregards the attachment itself along with the filename.</p>\r\n\r\n<p style="font-size:small;-webkit-text-size-adjust:none;color:#666;">&mdash;<br />You are receiving this because you are subscribed to this thread.<br />Reply to this email directly, <a href="https://github.com/ikvk/imap_tools/issues/4">view it on GitHub</a>, or <a href="https://github.com/notifications/unsubscribe-auth/AAV6N2ZD2P5MW33322GDAFTPSWDNHANCNFSM4HI6JQUA">mute the thread</a>.<img src="https://github.com/notifications/beacon/AAV6N27JEB4SBCE3VILPO6LPSWDNHANCNFSM4HI6JQUA.gif" height="1" width="1" alt="" /></p>\r\n<script type="application/json" data-scope="inboxmarkup">{"api_version":"1.0","publisher":{"api_key":"05dde50f1d1a384dd78767c55493e4bb","name":"GitHub"},"entity":{"external_key":"github/ikvk/imap_tools","title":"ikvk/imap_tools","subtitle":"GitHub repository","main_image_url":"https://github.githubassets.com/images/email/message_cards/header.png","avatar_image_url":"https://github.githubassets.com/images/email/message_cards/avatar.png","action":{"name":"Open in GitHub","url":"https://github.com/ikvk/imap_tools"}},"updates":{"snippets":[{"icon":"DESCRIPTION","message":"does not extract attached EML files (#4)"}],"action":{"name":"View Issue","url":"https://github.com/ikvk/imap_tools/issues/4"}}}</script>\r\n<script type="application/ld+json">[\r\n{\r\n"@context": "http://schema.org",\r\n"@type": "EmailMessage",\r\n"potentialAction": {\r\n"@type": "ViewAction",\r\n"target": "https://github.com/ikvk/imap_tools/issues/4",\r\n"url": "https://github.com/ikvk/imap_tools/issues/4",\r\n"name": "View Issue"\r\n},\r\n"description": "View this Issue on GitHub",\r\n"publisher": {\r\n"@type": "Organization",\r\n"name": "GitHub",\r\n"url": "https://github.com"\r\n}\r\n}\r\n]</script>\r\n----==_mimepart_5cc58f537542_49eb3fba40ccd9603084be--\r\n',
        ),
        ],
    from_values=EmailAddress(name='Каукин Владимир', email='kaukinvk@yandex.ru'),
    to_values=(EmailAddress(name='', email='imap.tools@ya.ru'),),
    cc_values=(),
    bcc_values=(),
    reply_to_values=(),
)


import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='Re: We will help you refinance your home.',
    from_='abhijit.862153drinnan@datavalet.com',
    to=('cvs@bruce-guenter.dyndns.org',),
    cc=('rait@bruce-guenter.dyndns.org',),
    bcc=(),
    reply_to=('abhijit.862153drinnan@datavalet.com',),
    date=datetime.datetime(2005, 5, 6, 15, 55, 1, tzinfo=datetime.timezone(datetime.timedelta(seconds=14400))),
    date_str='Fri, 06 May 2005 15:55:01 +0400',
    text='',
    html='Hello,<p>\r\n\r\nYou have qualified for the lowest rate in years.<br>\r\nYou could get over $400,000 for as little as $500 a month.<br>\r\nLow rates are fixed no matter what.<p>\r\n\r\nPlease visit the link below to verify your information:<br>\r\n<a href="http://www.lenxzc.com/index2.php?refid=malx">Approval Form</a><p>\r\n\r\nBest Regards,<br>\r\nchianfong cuthbert, Account Manager<br>\r\nReynolds Associates, LLC<p>\r\n<p>\r\n--------------------<br>\r\nif you received this in error: <a href="http://www.lenxzc.com/r.php">re-m0-ve</a>\r\n',
    headers={'return-path': ('<hzwfrkq@hintontile.com>',), 'delivered-to': ('rait@bruce-guenter.dyndns.org',), 'received': ('(qmail 12977 invoked from network); 6 May 2005 10:58:40 -0000', 'from localhost (localhost [127.0.0.1])\r\n  by bruce-guenter.dyndns.org ([192.168.1.3]); 06 May 2005 10:58:40 -0000', 'from zak.futurequest.net ([127.0.0.1])\r\n  by localhost ([127.0.0.1])\r\n  with SMTP via TCP; 06 May 2005 10:58:40 -0000', '(qmail 2252 invoked from network); 6 May 2005 10:58:39 -0000', 'from lsne-catv-dhcp-29-115.urbanet.ch (unknown [80.238.29.115])\r\n  by zak.futurequest.net ([69.5.6.152])\r\n  with SMTP via TCP; 06 May 2005 10:58:29 -0000', 'from mail.datavalet.com (80.238.29.115)\r\n          by 80.238.29.115 (fairwayv.4) with SMTP\r\n          id <09166i58p>\r\n          (Authid: 3811297); Fri, 06 May 2005 10:52:01 -0100'), 'reply-to': ('"chianfong cuthbert" <abhijit.862153drinnan@datavalet.com>',), 'from': ('"chianfong cuthbert" <abhijit.862153drinnan@datavalet.com>',), 'to': ('cvs@bruce-guenter.dyndns.org',), 'cc': ('rait@bruce-guenter.dyndns.org',), 'subject': ('Re: We will help you refinance your home.',), 'date': ('Fri, 06 May 2005 15:55:01 +0400',), 'mime-version': ('1.0',), 'content-type': ('multipart/alternative;\r\n\tboundary="--651790_269334.01570"',), 'content-length': ('636',), 'lines': ('22',)},
    attachments=[],
    from_values=EmailAddress(name='chianfong cuthbert', email='abhijit.862153drinnan@datavalet.com'),
    to_values=(EmailAddress(name='', email='cvs@bruce-guenter.dyndns.org'),),
    cc_values=(EmailAddress(name='', email='rait@bruce-guenter.dyndns.org'),),
    bcc_values=(),
    reply_to_values=(EmailAddress(name='chianfong cuthbert', email='abhijit.862153drinnan@datavalet.com'),),
)


import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='Testing 123',
    from_='test@lindsaar.net',
    to=('raasdnil@gmail.com',),
    cc=(),
    bcc=(),
    reply_to=(),
    date=datetime.datetime(2008, 11, 22, 15, 4, 59, tzinfo=datetime.timezone(datetime.timedelta(seconds=39600))),
    date_str='Sat, 22 Nov 2008 15:04:59 +1100',
    text='Plain email.\r\n\r\nHope it works well!\r\n\r\nMikel\r\n',
    html='',
    headers={'delivered-to': ('raasdnil@gmail.com',), 'received': ('by 10.140.178.13 with SMTP id a13cs354079rvf;\r\n        Fri, 21 Nov 2008 20:05:05 -0800 (PST)', 'by 10.151.44.15 with SMTP id w15mr2254748ybj.98.1227326704711;\r\n        Fri, 21 Nov 2008 20:05:04 -0800 (PST)', 'from mail11.tpgi.com.au (mail11.tpgi.com.au [203.12.160.161])\r\n        by mx.google.com with ESMTP id 10si5117885gxk.81.2008.11.21.20.05.03;\r\n        Fri, 21 Nov 2008 20:05:04 -0800 (PST)', 'from [192.0.0.253] (60-241-138-146.static.tpgi.com.au [60.0.0.146])\r\n\tby mail11.tpgi.com.au (envelope-from test@lindsaar.net) (8.14.3/8.14.3) with ESMTP id mAM44xew022221\r\n\tfor <raasdnil@gmail.com>; Sat, 22 Nov 2008 15:05:01 +1100'), 'return-path': ('<test@lindsaar.net>',), 'received-spf': ('neutral (google.com: 203.12.160.161 is neither permitted nor denied by domain of test@lindsaar.net) client-ip=203.12.160.161;',), 'authentication-results': ('mx.google.com; spf=neutral (google.com: 203.12.160.161 is neither permitted nor denied by domain of test@lindsaar.net) smtp.mail=test@lindsaar.net',), 'x-tpg-junk-status': ('Message not scanned',), 'x-tpg-antivirus': ('Passed',), 'message-id': ('<6B7EC235-5B17-4CA8-B2B8-39290DEB43A3@test.lindsaar.net>',), 'from': ('Mikel Lindsaar <test@lindsaar.net>',), 'to': ('Mikel Lindsaar <raasdnil@gmail.com>',), 'content-type': ('text/plain; charset=US-ASCII; format=flowed',), 'content-transfer-encoding': ('7bit',), 'mime-version': ('1.0 (Apple Message framework v929.2)',), 'subject': ('Testing 123',), 'date': ('Sat, 22 Nov 2008 15:04:59 +1100',), 'x-mailer': ('Apple Mail (2.929.2)',)},
    attachments=[],
    from_values=EmailAddress(name='Mikel Lindsaar', email='test@lindsaar.net'),
    to_values=(EmailAddress(name='Mikel Lindsaar', email='raasdnil@gmail.com'),),
    cc_values=(),
    bcc_values=(),
    reply_to_values=(),
)


import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='Returned mail: see transcript for details',
    from_='MAILER-DAEMON@tttttt.com.au',
    to=('mikel@sssss.net',),
    cc=(),
    bcc=(),
    reply_to=(),
    date=datetime.datetime(2007, 12, 24, 10, 3, 53, tzinfo=datetime.timezone(datetime.timedelta(seconds=39600))),
    date_str='Mon, 24 Dec 2007 10:03:53 +1100',
    text='The original message was received at Mon, 24 Dec 2007 10:03:47 +1100\r\nfrom 60-0-0-146.static.tttttt.com.au [60.0.0.146]\r\n\r\nThis message was generated by mail12.tttttt.com.au\r\n\r\n   ----- The following addresses had permanent fatal errors -----\r\n<edwin@zzzzzzz.com>\r\n    (reason: 553 5.3.0 <edwin@zzzzzzz.com>... Unknown E-Mail Address)\r\n\r\n   ----- Transcript of session follows -----\r\n... while talking to mail.zzzzzz.com.:\r\n>>> DATA\r\n<<< 553 5.3.0 <edwin@zzzzzzz.com>... Unknown E-Mail Address\r\n550 5.1.1 <edwin@zzzzzzz.com>... User unknown\r\n<<< 503 5.0.0 Need RCPT (recipient)\r\n\r\n-- \r\nThis message has been scanned for viruses and\r\ndangerous content by MailScanner, and is\r\nbelieved to be clean.\r\n\r\n',
    html='',
    headers={'return-path': ('<MAILER-DAEMON@imap01.sssss.net>',), 'received': ('from acomputer ([unix socket])\r\n\t by imap01.sssss.net (Cyrus) with LMTPA;\r\n\t Sun, 23 Dec 2007 15:04:04 -0800', 'from smtp.sssss.org (unknown [198.0.0.92])\r\n\tby imap01.sssss.net (Postfix) with ESMTP id E434877FB0\r\n\tfor <mikel@sssss.net>; Sun, 23 Dec 2007 15:04:03 -0800 (PST)', 'from mail12.tttttt.com.au (mail12.tttttt.com.au [203.0.0.162])\r\n\tby smtp.sssss.org (Postfix) with ESMTP id 53313B42B7\r\n\tfor <mikel@sssss.net>; Sun, 23 Dec 2007 15:03:54 -0800 (PST)', 'from localhost (localhost)\r\n\tby mail12.tttttt.com.au (envelope-from MAILER-DAEMON) (8.14.2/8.14.2) id lBNN3rDp003436;\r\n\tMon, 24 Dec 2007 10:03:53 +1100'), 'x-sieve': ('CMU Sieve 2.2',), 'date': ('Mon, 24 Dec 2007 10:03:53 +1100',), 'from': ('Mail Delivery Subsystem <MAILER-DAEMON@tttttt.com.au>',), 'message-id': ('<200712232303.lBNN3rDp003436@mail12.rrrr.com.au>',), 'to': ('<mikel@sssss.net>',), 'mime-version': ('1.0',), 'content-type': ('multipart/report; report-type=delivery-status;\r\n\tboundary="lBNN3rDp003436.1198451033/mail12.rrrr.com.au"',), 'subject': ('Returned mail: see transcript for details',), 'auto-submitted': ('auto-generated (failure)',), 'x-spam-status': ('No',)},
    attachments=[],
    from_values=EmailAddress(name='Mail Delivery Subsystem', email='MAILER-DAEMON@tttttt.com.au'),
    to_values=(EmailAddress(name='', email='mikel@sssss.net'),),
    cc_values=(),
    bcc_values=(),
    reply_to_values=(),
)


import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='worse when you use them.',
    from_='xxxxx@xxxxx',
    to=('',),
    cc=(),
    bcc=(),
    reply_to=(),
    date=datetime.datetime(2005, 4, 27, 14, 15, 31, tzinfo=datetime.timezone(datetime.timedelta(days=-1, seconds=61200))),
    date_str='Wed, 27 Apr 2005 14:15:31 -0700',
    text='\r\nXXXXX Xxxxx\r\n',
    html='',
    headers={'mime-version': ('1.0 (Apple Message framework v619.2)',), 'to': ('"xxxxx@xxxxx" <matmail>',), 'message-id': ('<416eaebec6d333ec6939eaf8a7d80724@xxxxx>',), 'content-type': ('multipart/alternative;\r\n        boundary=Apple-Mail-5-1037861608',), 'from': ('"xxxxx@xxxxx" <xxxxx@xxxxx>',), 'subject': ('worse when you use them.',), 'date': ('Wed, 27 Apr 2005 14:15:31 -0700',)},
    attachments=[],
    from_values=EmailAddress(name='xxxxx@xxxxx', email='xxxxx@xxxxx'),
    to_values=(EmailAddress(name='xxxxx@xxxxx', email=''),),
    cc_values=(),
    bcc_values=(),
    reply_to_values=(),
)


import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='Low or High Blood Pressure? We can help.',
    from_='fyouizjnp@swissonline.ch',
    to=('bait-passover@em.ca',),
    cc=(),
    bcc=(),
    reply_to=(),
    date=datetime.datetime(2004, 11, 7, 18, 23, 56, tzinfo=datetime.timezone(datetime.timedelta(days=-1, seconds=75600))),
    date_str='Sun, 07 Nov 2004 18:23:56 -0300',
    text="If you're in need of a good RX site for online purchases, we are your answer.\r\n\r\nWith Tens of Thousands of happy customers who saved huge, you can't go wrong.\r\n\r\nhttp://magyar8stator.com/26 Lots More Info Here\r\n\r\nAbove URL is for more info & if you are interested.",
    html='If you\'re in need of a good RX site for online purchases, we are your answer.<br>\r\n<br>\r\nWith Tens of Thousands of happy customers who saved huge, you can\'t go wrong.<br>\r\n<br>\r\n<a href="http://magyar8stator.com/26">Lots More Info Here</a><br>\r\n<br>\r\n<br>\r\nAbove URL is for more info & if you are interested.<br>\r\n\r\n\r\n\r\n',
    headers={'return-path': ('<fyouizjnp@swissonline.ch>',), 'delivered-to': ('em-ca-bait-precedes@em.ca',), 'received': ('(qmail 7622 invoked by uid 115); 7 Nov 2004 21:30:59 -0000', 'from fyouizjnp@swissonline.ch by churchill by uid 64011 with qmail-scanner-1.22 \r\n (clamdscan: 0.75-1. spamassassin: 2.63.  Clear:RC:0(222.47.112.31):. \r\n Processed in 1.163709 secs); 07 Nov 2004 21:30:59 -0000', 'from unknown (HELO 209.5.178.248) (222.47.112.31)\r\n  by churchill.factcomp.com with SMTP; 7 Nov 2004 21:30:57 -0000'), 'message-id': ('<qlrqpxforpx$RND_WORDjyeottngpx@sympatico.ca>',), 'from': ('"Wilmer Capps" <fyouizjnp@swissonline.ch>',), 'to': ('bait-passover@em.ca',), 'subject': ('Low or High Blood Pressure? We can help.',), 'date': ('Sun, 07 Nov 2004 18:23:56 -0300',), 'x-mailer': ('gene annulus',), 'pneumococcus-polyhedral': ('belgrade hanoverian lima',), 'mime-version': ('1.0',), 'content-type': ('multipart/alternative;\r\n           boundary="----=_NextPart_000_003A_5008K66J.3Z4LC80J"',)},
    attachments=[],
    from_values=EmailAddress(name='Wilmer Capps', email='fyouizjnp@swissonline.ch'),
    to_values=(EmailAddress(name='', email='bait-passover@em.ca'),),
    cc_values=(),
    bcc_values=(),
    reply_to_values=(),
)


import re
from typing import Optional, Iterable, List, Dict, Tuple

from .imap_utf7 import utf7_decode
from .consts import MailBoxFolderStatusOptions
from .utils import check_command_status, pairs_to_dict, encode_folder, StrOrBytes
from .errors import MailboxFolderStatusValueError, MailboxFolderSelectError, MailboxFolderCreateError, \
    MailboxFolderRenameError, MailboxFolderDeleteError, MailboxFolderStatusError, MailboxFolderSubscribeError


class FolderInfo:
    """
    Mailbox folder info
        name: str - folder name
        delim: str - delimiter, a character used to delimit levels of hierarchy in a mailbox name
        flags: (str,) - folder flags
    A 'NIL' delimiter means that no hierarchy exists, the name is a "flat" name.
    """
    __slots__ = 'name', 'delim', 'flags'

    def __init__(self, name: str, delim: str, flags: Tuple[str, ...]):
        self.name = name
        self.delim = delim
        self.flags = flags

    def __repr__(self):
        return f"{self.__class__.__name__}(name={repr(self.name)}, delim={repr(self.delim)}, flags={repr(self.flags)})"

    def __eq__(self, other):
        return all(getattr(self, i) == getattr(other, i) for i in self.__slots__)


class MailBoxFolderManager:
    """Operations with mailbox folders"""

    def __init__(self, mailbox):
        self.mailbox = mailbox
        self._current_folder = None

    def set(self, folder: StrOrBytes, readonly: bool = False) -> tuple:
        """Select current folder"""
        result = self.mailbox.client.select(encode_folder(folder), readonly)
        check_command_status(result, MailboxFolderSelectError)
        self._current_folder = folder
        return result

    def exists(self, folder: str) -> bool:
        """Checks whether a folder exists on the server."""
        return len(self.list('', folder)) > 0

    def create(self, folder: StrOrBytes) -> tuple:
        """
        Create folder on the server.
        Use email box delimiter to separate folders. Example for "|" delimiter: "folder|sub folder"
        """
        result = self.mailbox.client._simple_command('CREATE', encode_folder(folder))
        check_command_status(result, MailboxFolderCreateError)
        return result

    def get(self) -> Optional[str]:
        """
        Get current folder
        :return:
            None - if folder is not selected
            str - if folder is selected
        """
        return self._current_folder

    def rename(self, old_name: StrOrBytes, new_name: StrOrBytes) -> tuple:
        """Rename folder from old_name to new_name"""
        result = self.mailbox.client._simple_command(
            'RENAME', encode_folder(old_name), encode_folder(new_name))
        check_command_status(result, MailboxFolderRenameError)
        return result

    def delete(self, folder: StrOrBytes) -> tuple:
        """Delete folder"""
        result = self.mailbox.client._simple_command('DELETE', encode_folder(folder))
        check_command_status(result, MailboxFolderDeleteError)
        return result

    def status(self, folder: Optional[StrOrBytes] = None, options: Optional[Iterable[str]] = None) -> Dict[str, int]:
        """
        Get the status of a folder
        :param folder: mailbox folder, current folder if None
        :param options: [str] with values from MailBoxFolderStatusOptions.all | None - for get all options
        :return: dict with available options keys
            example: {'MESSAGES': 41, 'RECENT': 0, 'UIDNEXT': 11996, 'UIDVALIDITY': 1, 'UNSEEN': 5}
        """
        command = 'STATUS'
        if folder is None:
            folder = self.get()
        if not options:
            options = tuple(MailBoxFolderStatusOptions.all)
        for opt in options:
            if opt not in MailBoxFolderStatusOptions.all:
                raise MailboxFolderStatusValueError(str(opt))
        status_result = self.mailbox.client._simple_command(
            command, encode_folder(folder), f'({" ".join(options)})')
        check_command_status(status_result, MailboxFolderStatusError)
        result = self.mailbox.client._untagged_response(status_result[0], status_result[1], command)
        check_command_status(result, MailboxFolderStatusError)
        status_data = [i for i in result[1] if type(i) is bytes][0]  # may contain tuples with encoded names
        values = status_data.decode().split('(')[-1].split(')')[0].split(' ')
        return {k: int(v) for k, v in pairs_to_dict(values).items() if str(v).isdigit()}

    def list(self, folder: StrOrBytes = '', search_args: str = '*', subscribed_only: bool = False) -> List[FolderInfo]:
        """
        Get a listing of folders on the server
        :param folder: mailbox folder, if empty - get from root
        :param search_args: search arguments, is case-sensitive mailbox name with possible wildcards
            * is a wildcard, and matches zero or more characters at this position
            % is similar to * but it does not match a hierarchy delimiter
        :param subscribed_only: bool - get only subscribed folders
        :return: [FolderInfo]
        """
        folder_item_re = re.compile(r'\((?P<flags>[\S ]*?)\) (?P<delim>[\S]+) (?P<name>.+)')
        command = 'LSUB' if subscribed_only else 'LIST'
        typ, data = self.mailbox.client._simple_command(
            command, encode_folder(folder), encode_folder(search_args))
        typ, data = self.mailbox.client._untagged_response(typ, data, command)
        result = []
        for folder_item in data:
            if not folder_item:
                continue
            if type(folder_item) is bytes:
                folder_match = re.search(folder_item_re, utf7_decode(folder_item))
                if not folder_match:
                    continue
                folder_dict = folder_match.groupdict()
                name = folder_dict['name']
                if name.startswith('"') and name.endswith('"'):
                    name = name[1:-1]
            elif type(folder_item) is tuple:
                # when name has " or \ chars
                folder_match = re.search(folder_item_re, utf7_decode(folder_item[0]))
                if not folder_match:
                    continue
                folder_dict = folder_match.groupdict()
                name = utf7_decode(folder_item[1])
            else:
                continue
            result.append(FolderInfo(
                name=name.replace('\\"', '"'),
                delim=folder_dict['delim'].replace('"', ''),
                flags=tuple(folder_dict['flags'].split())  # noqa,
            ))
        return result

    def subscribe(self, folder: StrOrBytes, value: bool) -> tuple:
        """subscribe/unsubscribe to folder"""
        method = self.mailbox.client.subscribe if value else self.mailbox.client.unsubscribe
        result = method(encode_folder(folder))
        check_command_status(result, MailboxFolderSubscribeError)
        return result



import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='S19 IZM A7-342',
    from_='k1@yandex.ru',
    to=('rasp@wat.aero',),
    cc=('aa@com.ru', 'mm@com.ru'),
    bcc=('hello@yandex-team.ru',),
    reply_to=(),
    date=datetime.datetime(2019, 2, 7, 13, 18, 20, tzinfo=datetime.timezone(datetime.timedelta(seconds=18000))),
    date_str='Thu, 7 Feb 2019 13:18:20 +0500',
    text='SCR\r\n\r\n/\r\nS19\r\n07FEB\r\nwat\r\n\r\n-- \r\nотдел расписания\r\nтел. (343) 072-00-00 (вн.18-59)\r\ne-mail: e.sp@com.ru\r\n\r\nДобрый\xa0день,\xa0уважаемые\xa0коллеги!\r\n\r\n\r\n\r\n\r\n-------- Перенаправленное сообщение --------\r\nТема: \tВвод рейсов ТЮМ, ЧЛБ, ОМС на Лето 2019\r\nДата: \tFri, 7 Dec 2018 08:23:21 +0500\r\nОт: \tПлёткина <a.pl@com.ru>\r\nКому: \tБелобородова О.Н. <schedule@com.ru>\r\n\r\n\r\n\r\nДобрый\xa0день,\xa0уважаемые\xa0коллеги!\r\n\r\nПрошу ввести рейс на период летней навигации\r\n\r\n*ДМД-ТЮМ-ДМД \xa0 У6-341/342*\xa0 тип ВС А320\r\n\r\nВВ из ДМД 23:00 LT. ВВ из ТЮМ 06:45 LT\r\n\r\nисключение период с 25/05/19 по 13/10/19\r\n\r\nВВ из ДМД в 00:50 LT\r\n\r\nВВ из ТЮМ остается без изменения\r\n\r\n----------------------------------------\r\n\r\n*ДМД-ОМС-ДМД \xa0 У6-387/388*\xa0 тип ВС А321\r\n\r\nВВ из ДМД в 23:15. ВВ из ОМС 06:30 LT\r\n\r\n----------------------------------------\r\n\r\n*ДМД-ЧЛБ-ДМД \xa0 У6-610/609*\xa0 тип ВС А320\r\n\r\nВВ из ДМД в 23:30. ВВ из ЧЛБ 06:40 LT\r\n\r\n\r\n',
    html='<html>\r\n  <head>\r\n\r\n    <meta http-equiv="content-type" content="text/html; charset=utf-8">\r\n  </head>\r\n  <body text="#000000" bgcolor="#FFFFFF">\r\n    <p>SCR<br>\r\n    </p>\r\n    <div class="moz-forward-container">/<br>\r\n      S19<br>\r\n      07FEB<br>\r\n      wat<br></div>\r\n    <pre class="moz-signature" cols="72">-- \r\nотдел расписания\r\nтел. (343) 072-00-00 (вн.18-59)\r\ne-mail: <a class="moz-txt-link-abbreviated" href="mailto:e.sp@com.ru">e.sp@com.ru</a></pre>\r\n  </body>\r\n</html>\r\n<html>\r\n  <head>\r\n\r\n    <meta http-equiv="content-type" content="text/html; charset=utf-8">\r\n  </head>\r\n  <body smarttemplateinserted="true">\r\n    <div id="smartTemplate4-template">Добрый\xa0день,\xa0уважаемые\xa0коллеги!\r\n      <p>\xa0</p>\r\n    </div>\r\n    <br>\r\n    <div class="moz-forward-container"><br>\r\n      <br>\r\n      -------- Перенаправленное сообщение --------\r\n      <table class="moz-email-headers-table" cellspacing="0"\r\n        cellpadding="0" border="0">\r\n        <tbody>\r\n          <tr>\r\n            <th valign="BASELINE" nowrap="nowrap" align="RIGHT">Тема: </th>\r\n            <td>Ввод рейсов ТЮМ, ЧЛБ, ОМС на Лето 2019</td>\r\n          </tr>\r\n          <tr>\r\n            <th valign="BASELINE" nowrap="nowrap" align="RIGHT">Дата: </th>\r\n            <td>Fri, 7 Dec 2018 08:23:21 +0500</td>\r\n          </tr>\r\n          <tr>\r\n            <th valign="BASELINE" nowrap="nowrap" align="RIGHT">От: </th>\r\n            <td>1 <a class="moz-txt-link-rfc2396E" href="mailto:a.pl@com.ru">&lt;a.pl@com.ru&gt;</a></td>\r\n          </tr>\r\n          <tr>\r\n            <th valign="BASELINE" nowrap="nowrap" align="RIGHT">Кому: </th>\r\n            <td>2 <a class="moz-txt-link-rfc2396E" href="mailto:schedule@com.ru">&lt;schedule@com.ru&gt;</a></td>\r\n          </tr>\r\n        </tbody>\r\n      </table>\r\n      <br>\r\n      <br>\r\n      <meta http-equiv="content-type" content="text/html; charset=utf-8">\r\n      <div id="smartTemplate4-template">Добрый\xa0день,\xa0уважаемые\xa0коллеги!\r\n        <p>Прошу ввести рейс на период летней навигации <br>\r\n        </p>\r\n        <p>\xa0 тип ВС А320<br>\r\n        </p>\r\n        <p>ВВ из ДМД 23:00 LT. ВВ из ТЮМ 06:45 LT</p>\r\n        <p>исключение период с 25/05/19 по 13/10/19 <br>\r\n        </p>\r\n        <p>ВВ из ДМД в 00:50 LT<br>\r\n        </p>\r\n        <p>ВВ из ТЮМ остается без изменения <br>\r\n        </p>\r\n        <p>----------------------------------------</p>\r\n        <p>\xa0 тип ВС А321</p>\r\n        <p>ВВ из ДМД в 23:15. ВВ из ОМС 06:30 LT <br>\r\n        </p>\r\n        <p>----------------------------------------</p>\r\n        <p>\xa0 тип ВС А320</p>\r\n        <p>ВВ из ДМД в 23:30. ВВ из ЧЛБ 06:40 LT </p>\r\n      </div>\r\n      <br>\r\n    </div>\r\n  </body>\r\n</html>\r\n',
    headers={'return-path': ('e.sp@com.ru',), 'received': ('from m101.com.ru (LHLO m101.com.ru) (92.233.222.111) by m101.com.ru\r\n with LMTP; Thu, 7 Feb 2019 13:18:22 +0500 (YEKT)', 'from localhost (localhost [127.0.0.1])\r\n\tby m101.com.ru (Postfix) with ESMTP id 9A9181170B7;\r\n\tThu,  7 Feb 2019 13:18:22 +0500 (YEKT)', 'from m101.com.ru ([127.0.0.1])\r\n\tby localhost (m101.com.ru [127.0.0.1]) (amavisd-new, port 10026)\r\n\twith ESMTP id Q3OLc4gNDjjO; Thu,  7 Feb 2019 13:18:22 +0500 (YEKT)', 'from [192.168.104.100] (kom-420-1.hades.company [192.168.104.100])\r\n\tby m101.com.ru (Postfix) with ESMTPSA id DC7BB1170A1;\r\n\tThu,  7 Feb 2019 13:18:21 +0500 (YEKT)'), 'x-virus-scanned': ('amavisd-new at m101.com.ru',), 'subject': ('S19 IZM A7-342',), 'references': ('<1333139463.20190207083243@com.ru>',), 'to': ('=?UTF-8?B?0KLQrtCc0JXQndCs?= <rasp@wat.aero>',), 'from': ('=?utf-8?B?0JrQsNGD0LrQuNC9INCS0LvQsNC00LjQvNC40YA=?= <k1@yandex.ru>',), 'cc': ('aa@com.ru, mm@com.ru',), 'bcc': ('=?utf-8?b?0JrQvtC80LDQvdC00LAg0K/QvdC00LXQutGBLtCf0L7Rh9GC0Ys=?=\r\n\t<hello@yandex-team.ru>',), 'x-forwarded-message-id': ('<1333139463.20190207083243@com.ru>',), 'message-id': ('<2ffd4cec-d312-60da-72f3-5b6e4406fddf@com.ru>',), 'date': ('Thu, 7 Feb 2019 13:18:20 +0500',), 'user-agent': ('Mozilla/5.0 (Windows NT 10.0; WOW64; rv:60.0) Gecko/20100101\r\n Thunderbird/60.5.0',), 'mime-version': ('1.0',), 'in-reply-to': ('<1333139463.20190207083243@com.ru>',), 'content-type': ('multipart/mixed;\r\n boundary="------------FFFBFD561B1FF33B4E5E7050"',), 'content-language': ('ru',)},
    attachments=[
        dict(
            filename='Message01.eml',
            content_id='',
            content_disposition='attachment',
            content_type='message/rfc822',
            payload=b'Return-Path: a.pl@com.ru\nReceived: from m101.com.ru (LHLO m101.com.ru) (92.233.222.111) by m101.com.ru\n with LMTP; Thu, 7 Feb 2019 08:29:46 +0500 (YEKT)\nReceived: from localhost (localhost [127.0.0.1])\n\tby m101.com.ru (Postfix) with ESMTP id 5C11A78A70;\n\tThu,  7 Feb 2019 08:29:46 +0500 (YEKT)\nX-Virus-Scanned: amavisd-new at m101.com.ru\nReceived: from m101.com.ru ([127.0.0.1])\n\tby localhost (m101.com.ru [127.0.0.1]) (amavisd-new, port 10026)\n\twith ESMTP id qPDszqYc1_Us; Thu,  7 Feb 2019 08:29:46 +0500 (YEKT)\nReceived: from [192.168.104.53] (kom-419-31.hades.company [192.168.104.53])\n\tby m101.com.ru (Postfix) with ESMTPSA id 29D6C78A67\n\tfor <a.nov@company.ru>; Thu,  7 Feb 2019 08:29:46 +0500 (YEKT)\nSubject: S19 IZM A7-342\nReferences: <4a598974-6731-c4f3-0efa-2f1e2b044f08@com.ru>\nTo: Alla Nov <a.nov@company.ru>\nFrom: =?utf-8?B?0JrQsNGD0LrQuNC9INCS0LvQsNC00LjQvNC40YA=?= <k1@yandex.ru>\nX-Forwarded-Message-Id: <4a598974-6731-c4f3-0efa-2f1e2b044f08@com.ru>\nMessage-ID: <09ccd226-f9e9-77e0-e456-3490e024d16e@com.ru>\nDate: Thu, 7 Feb 2019 08:29:45 +0500\nUser-Agent: Mozilla/5.0 (Windows NT 10.0; WOW64; rv:60.0) Gecko/20100101\n Thunderbird/60.4.0\nMIME-Version: 1.0\nIn-Reply-To: <4a598974-6731-c4f3-0efa-2f1e2b044f08@com.ru>\nContent-Type: multipart/alternative;\n boundary="------------CC10C5F7DB2594C21A5E2BDB"\nContent-Language: ru\n\n--------------CC10C5F7DB2594C21A5E2BDB\nContent-Type: text/plain; charset=utf-8; format=flowed\nContent-Transfer-Encoding: 8bit\n\n\xd0\x94\xd0\xbe\xd0\xb1\xd1\x80\xd1\x8b\xd0\xb9\xc2\xa0\xd0\xb4\xd0\xb5\xd0\xbd\xd1\x8c,\xc2\xa0\xd1\x83\xd0\xb2\xd0\xb0\xd0\xb6\xd0\xb0\xd0\xb5\xd0\xbc\xd1\x8b\xd0\xb5\xc2\xa0\xd0\xba\xd0\xbe\xd0\xbb\xd0\xbb\xd0\xb5\xd0\xb3\xd0\xb8!\n\n\n\n\n-------- \xd0\x9f\xd0\xb5\xd1\x80\xd0\xb5\xd0\xbd\xd0\xb0\xd0\xbf\xd1\x80\xd0\xb0\xd0\xb2\xd0\xbb\xd0\xb5\xd0\xbd\xd0\xbd\xd0\xbe\xd0\xb5 \xd1\x81\xd0\xbe\xd0\xbe\xd0\xb1\xd1\x89\xd0\xb5\xd0\xbd\xd0\xb8\xd0\xb5 --------\n\xd0\xa2\xd0\xb5\xd0\xbc\xd0\xb0: \t\xd0\x92\xd0\xb2\xd0\xbe\xd0\xb4 \xd1\x80\xd0\xb5\xd0\xb9\xd1\x81\xd0\xbe\xd0\xb2 \xd0\xa2\xd0\xae\xd0\x9c, \xd0\xa7\xd0\x9b\xd0\x91, \xd0\x9e\xd0\x9c\xd0\xa1 \xd0\xbd\xd0\xb0 \xd0\x9b\xd0\xb5\xd1\x82\xd0\xbe 2019\n\xd0\x94\xd0\xb0\xd1\x82\xd0\xb0: \tFri, 7 Dec 2018 08:23:21 +0500\n\xd0\x9e\xd1\x82: \t\xd0\x9f\xd0\xbb\xd1\x91\xd1\x82\xd0\xba\xd0\xb8\xd0\xbd\xd0\xb0 <a.pl@com.ru>\n\xd0\x9a\xd0\xbe\xd0\xbc\xd1\x83: \t\xd0\x91\xd0\xb5\xd0\xbb\xd0\xbe\xd0\xb1\xd0\xbe\xd1\x80\xd0\xbe\xd0\xb4\xd0\xbe\xd0\xb2\xd0\xb0 \xd0\x9e.\xd0\x9d. <schedule@com.ru>\n\n\n\n\xd0\x94\xd0\xbe\xd0\xb1\xd1\x80\xd1\x8b\xd0\xb9\xc2\xa0\xd0\xb4\xd0\xb5\xd0\xbd\xd1\x8c,\xc2\xa0\xd1\x83\xd0\xb2\xd0\xb0\xd0\xb6\xd0\xb0\xd0\xb5\xd0\xbc\xd1\x8b\xd0\xb5\xc2\xa0\xd0\xba\xd0\xbe\xd0\xbb\xd0\xbb\xd0\xb5\xd0\xb3\xd0\xb8!\n\n\xd0\x9f\xd1\x80\xd0\xbe\xd1\x88\xd1\x83 \xd0\xb2\xd0\xb2\xd0\xb5\xd1\x81\xd1\x82\xd0\xb8 \xd1\x80\xd0\xb5\xd0\xb9\xd1\x81 \xd0\xbd\xd0\xb0 \xd0\xbf\xd0\xb5\xd1\x80\xd0\xb8\xd0\xbe\xd0\xb4 \xd0\xbb\xd0\xb5\xd1\x82\xd0\xbd\xd0\xb5\xd0\xb9 \xd0\xbd\xd0\xb0\xd0\xb2\xd0\xb8\xd0\xb3\xd0\xb0\xd1\x86\xd0\xb8\xd0\xb8\n\n*\xd0\x94\xd0\x9c\xd0\x94-\xd0\xa2\xd0\xae\xd0\x9c-\xd0\x94\xd0\x9c\xd0\x94 \xc2\xa0 \xd0\xa36-341/342*\xc2\xa0 \xd1\x82\xd0\xb8\xd0\xbf \xd0\x92\xd0\xa1 \xd0\x90320\n\n\xd0\x92\xd0\x92 \xd0\xb8\xd0\xb7 \xd0\x94\xd0\x9c\xd0\x94 23:00 LT. \xd0\x92\xd0\x92 \xd0\xb8\xd0\xb7 \xd0\xa2\xd0\xae\xd0\x9c 06:45 LT\n\n\xd0\xb8\xd1\x81\xd0\xba\xd0\xbb\xd1\x8e\xd1\x87\xd0\xb5\xd0\xbd\xd0\xb8\xd0\xb5 \xd0\xbf\xd0\xb5\xd1\x80\xd0\xb8\xd0\xbe\xd0\xb4 \xd1\x81 25/05/19 \xd0\xbf\xd0\xbe 13/10/19\n\n\xd0\x92\xd0\x92 \xd0\xb8\xd0\xb7 \xd0\x94\xd0\x9c\xd0\x94 \xd0\xb2 00:50 LT\n\n\xd0\x92\xd0\x92 \xd0\xb8\xd0\xb7 \xd0\xa2\xd0\xae\xd0\x9c \xd0\xbe\xd1\x81\xd1\x82\xd0\xb0\xd0\xb5\xd1\x82\xd1\x81\xd1\x8f \xd0\xb1\xd0\xb5\xd0\xb7 \xd0\xb8\xd0\xb7\xd0\xbc\xd0\xb5\xd0\xbd\xd0\xb5\xd0\xbd\xd0\xb8\xd1\x8f\n\n----------------------------------------\n\n*\xd0\x94\xd0\x9c\xd0\x94-\xd0\x9e\xd0\x9c\xd0\xa1-\xd0\x94\xd0\x9c\xd0\x94 \xc2\xa0 \xd0\xa36-387/388*\xc2\xa0 \xd1\x82\xd0\xb8\xd0\xbf \xd0\x92\xd0\xa1 \xd0\x90321\n\n\xd0\x92\xd0\x92 \xd0\xb8\xd0\xb7 \xd0\x94\xd0\x9c\xd0\x94 \xd0\xb2 23:15. \xd0\x92\xd0\x92 \xd0\xb8\xd0\xb7 \xd0\x9e\xd0\x9c\xd0\xa1 06:30 LT\n\n----------------------------------------\n\n*\xd0\x94\xd0\x9c\xd0\x94-\xd0\xa7\xd0\x9b\xd0\x91-\xd0\x94\xd0\x9c\xd0\x94 \xc2\xa0 \xd0\xa36-610/609*\xc2\xa0 \xd1\x82\xd0\xb8\xd0\xbf \xd0\x92\xd0\xa1 \xd0\x90320\n\n\xd0\x92\xd0\x92 \xd0\xb8\xd0\xb7 \xd0\x94\xd0\x9c\xd0\x94 \xd0\xb2 23:30. \xd0\x92\xd0\x92 \xd0\xb8\xd0\xb7 \xd0\xa7\xd0\x9b\xd0\x91 06:40 LT\n\n\n\n--------------CC10C5F7DB2594C21A5E2BDB\nContent-Type: text/html; charset=utf-8\nContent-Transfer-Encoding: 8bit\n\n<html>\n  <head>\n\n    <meta http-equiv="content-type" content="text/html; charset=UTF-8">\n  </head>\n  <body smarttemplateinserted="true">\n    <div id="smartTemplate4-template">\xd0\x94\xd0\xbe\xd0\xb1\xd1\x80\xd1\x8b\xd0\xb9\xc2\xa0\xd0\xb4\xd0\xb5\xd0\xbd\xd1\x8c,\xc2\xa0\xd1\x83\xd0\xb2\xd0\xb0\xd0\xb6\xd0\xb0\xd0\xb5\xd0\xbc\xd1\x8b\xd0\xb5\xc2\xa0\xd0\xba\xd0\xbe\xd0\xbb\xd0\xbb\xd0\xb5\xd0\xb3\xd0\xb8!\n      <p>\xc2\xa0</p>\n    </div>\n    <br>\n    <div class="moz-forward-container"><br>\n      <br>\n      -------- \xd0\x9f\xd0\xb5\xd1\x80\xd0\xb5\xd0\xbd\xd0\xb0\xd0\xbf\xd1\x80\xd0\xb0\xd0\xb2\xd0\xbb\xd0\xb5\xd0\xbd\xd0\xbd\xd0\xbe\xd0\xb5 \xd1\x81\xd0\xbe\xd0\xbe\xd0\xb1\xd1\x89\xd0\xb5\xd0\xbd\xd0\xb8\xd0\xb5 --------\n      <table class="moz-email-headers-table" cellspacing="0"\n        cellpadding="0" border="0">\n        <tbody>\n          <tr>\n            <th valign="BASELINE" nowrap="nowrap" align="RIGHT">\xd0\xa2\xd0\xb5\xd0\xbc\xd0\xb0: </th>\n            <td>\xd0\x92\xd0\xb2\xd0\xbe\xd0\xb4 \xd1\x80\xd0\xb5\xd0\xb9\xd1\x81\xd0\xbe\xd0\xb2 \xd0\xa2\xd0\xae\xd0\x9c, \xd0\xa7\xd0\x9b\xd0\x91, \xd0\x9e\xd0\x9c\xd0\xa1 \xd0\xbd\xd0\xb0 \xd0\x9b\xd0\xb5\xd1\x82\xd0\xbe 2019</td>\n          </tr>\n          <tr>\n            <th valign="BASELINE" nowrap="nowrap" align="RIGHT">\xd0\x94\xd0\xb0\xd1\x82\xd0\xb0: </th>\n            <td>Fri, 7 Dec 2018 08:23:21 +0500</td>\n          </tr>\n          <tr>\n            <th valign="BASELINE" nowrap="nowrap" align="RIGHT">\xd0\x9e\xd1\x82: </th>\n            <td>1 <a class="moz-txt-link-rfc2396E" href="mailto:a.pl@com.ru">&lt;a.pl@com.ru&gt;</a></td>\n          </tr>\n          <tr>\n            <th valign="BASELINE" nowrap="nowrap" align="RIGHT">\xd0\x9a\xd0\xbe\xd0\xbc\xd1\x83: </th>\n            <td>2 <a class="moz-txt-link-rfc2396E" href="mailto:schedule@com.ru">&lt;schedule@com.ru&gt;</a></td>\n          </tr>\n        </tbody>\n      </table>\n      <br>\n      <br>\n      <meta http-equiv="content-type" content="text/html; charset=UTF-8">\n      <div id="smartTemplate4-template">\xd0\x94\xd0\xbe\xd0\xb1\xd1\x80\xd1\x8b\xd0\xb9\xc2\xa0\xd0\xb4\xd0\xb5\xd0\xbd\xd1\x8c,\xc2\xa0\xd1\x83\xd0\xb2\xd0\xb0\xd0\xb6\xd0\xb0\xd0\xb5\xd0\xbc\xd1\x8b\xd0\xb5\xc2\xa0\xd0\xba\xd0\xbe\xd0\xbb\xd0\xbb\xd0\xb5\xd0\xb3\xd0\xb8!\n        <p>\xd0\x9f\xd1\x80\xd0\xbe\xd1\x88\xd1\x83 \xd0\xb2\xd0\xb2\xd0\xb5\xd1\x81\xd1\x82\xd0\xb8 \xd1\x80\xd0\xb5\xd0\xb9\xd1\x81 \xd0\xbd\xd0\xb0 \xd0\xbf\xd0\xb5\xd1\x80\xd0\xb8\xd0\xbe\xd0\xb4 \xd0\xbb\xd0\xb5\xd1\x82\xd0\xbd\xd0\xb5\xd0\xb9 \xd0\xbd\xd0\xb0\xd0\xb2\xd0\xb8\xd0\xb3\xd0\xb0\xd1\x86\xd0\xb8\xd0\xb8 <br>\n        </p>\n        <p>\xc2\xa0 \xd1\x82\xd0\xb8\xd0\xbf \xd0\x92\xd0\xa1 \xd0\x90320<br>\n        </p>\n        <p>\xd0\x92\xd0\x92 \xd0\xb8\xd0\xb7 \xd0\x94\xd0\x9c\xd0\x94 23:00 LT. \xd0\x92\xd0\x92 \xd0\xb8\xd0\xb7 \xd0\xa2\xd0\xae\xd0\x9c 06:45 LT</p>\n        <p>\xd0\xb8\xd1\x81\xd0\xba\xd0\xbb\xd1\x8e\xd1\x87\xd0\xb5\xd0\xbd\xd0\xb8\xd0\xb5 \xd0\xbf\xd0\xb5\xd1\x80\xd0\xb8\xd0\xbe\xd0\xb4 \xd1\x81 25/05/19 \xd0\xbf\xd0\xbe 13/10/19 <br>\n        </p>\n        <p>\xd0\x92\xd0\x92 \xd0\xb8\xd0\xb7 \xd0\x94\xd0\x9c\xd0\x94 \xd0\xb2 00:50 LT<br>\n        </p>\n        <p>\xd0\x92\xd0\x92 \xd0\xb8\xd0\xb7 \xd0\xa2\xd0\xae\xd0\x9c \xd0\xbe\xd1\x81\xd1\x82\xd0\xb0\xd0\xb5\xd1\x82\xd1\x81\xd1\x8f \xd0\xb1\xd0\xb5\xd0\xb7 \xd0\xb8\xd0\xb7\xd0\xbc\xd0\xb5\xd0\xbd\xd0\xb5\xd0\xbd\xd0\xb8\xd1\x8f <br>\n        </p>\n        <p>----------------------------------------</p>\n        <p>\xc2\xa0 \xd1\x82\xd0\xb8\xd0\xbf \xd0\x92\xd0\xa1 \xd0\x90321</p>\n        <p>\xd0\x92\xd0\x92 \xd0\xb8\xd0\xb7 \xd0\x94\xd0\x9c\xd0\x94 \xd0\xb2 23:15. \xd0\x92\xd0\x92 \xd0\xb8\xd0\xb7 \xd0\x9e\xd0\x9c\xd0\xa1 06:30 LT <br>\n        </p>\n        <p>----------------------------------------</p>\n        <p>\xc2\xa0 \xd1\x82\xd0\xb8\xd0\xbf \xd0\x92\xd0\xa1 \xd0\x90320</p>\n        <p>\xd0\x92\xd0\x92 \xd0\xb8\xd0\xb7 \xd0\x94\xd0\x9c\xd0\x94 \xd0\xb2 23:30. \xd0\x92\xd0\x92 \xd0\xb8\xd0\xb7 \xd0\xa7\xd0\x9b\xd0\x91 06:40 LT </p>\n      </div>\n      <br>\n    </div>\n  </body>\n</html>\n\n--------------CC10C5F7DB2594C21A5E2BDB--\n\n',
        ),
        ],
    from_values=EmailAddress(name='Каукин Владимир', email='k1@yandex.ru'),
    to_values=(EmailAddress(name='ТЮМЕНЬ', email='rasp@wat.aero'),),
    cc_values=(EmailAddress(name='', email='aa@com.ru'), EmailAddress(name='', email='mm@com.ru')),
    bcc_values=(EmailAddress(name='Команда Яндекс.Почты', email='hello@yandex-team.ru'),),
    reply_to_values=(),
)


import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='Nicolas Fouché has accepted your invitation to Gmail',
    from_='gmail-noreply@google.com',
    to=('a.b@gmail.com',),
    cc=(),
    bcc=(),
    reply_to=('x.y@gmail.com',),
    date=datetime.datetime(2005, 6, 28, 1, 2, 11, tzinfo=datetime.timezone(datetime.timedelta(days=-1, seconds=61200))),
    date_str='Tue, 28 Jun 2005 01:02:11 -0700',
    text='Nicolas Fouché has accepted your invitation to Gmail and has chosen the \r\nbrand new address x.y@gmail.com. Be one of the first to email Nicolas \r\nat this new Gmail address--just hit reply and send Nicolas a message. \r\nx.y@gmail.com has also been automatically added to your contact list \r\nso you can stay in touch with Gmail. \r\n\r\n\r\nThanks, \r\n\r\nThe Gmail Team\r\n',
    html='<html>\r\n<font face="Arial, Helvetica, sans-serif">\r\n<p>Nicolas Fouché has accepted your invitation to Gmail and has\r\n  chosen the brand new address x.y@gmail.com. Be one of the first to email \r\n  Nicolas at this new Gmail address--just hit reply and send \r\n  Nicolas a message. x.y@gmail.com has also been automatically added to\r\n  your contact list so you can stay in touch with Gmail.\r\n</p>\r\n<p><br>\r\n  Thanks, </p>\r\n<p> The Gmail Team</p>\r\n</font>\r\n</html>\r\n',
    headers={'x-gmail-received': ('220984aec4c4885e060987be043c9363cbef8551',), 'received': ('by 10.36.47.16; Tue, 28 Jun 2005 01:02:11 -0700 (PDT)',), 'message-id': ('<89d7557c0506280102495d555f@mail.gmail.com>',), 'date': ('Tue, 28 Jun 2005 01:02:11 -0700',), 'from': ('Gmail Team <gmail-noreply@google.com>',), 'reply-to': ('x.y@gmail.com',), 'to': ('=?ISO-8859-1?Q?Nicolas_Fouch=E9?= <a.b@gmail.com>',), 'subject': ('=?ISO-8859-1?Q?Nicolas_Fouch=E9_has_accepted_your_invitation_to_Gmail?=',), 'mime-version': ('1.0',), 'content-type': ('multipart/alternative; \r\n\tboundary="----=_Part_976_15222032.1119945731186"',)},
    attachments=[],
    from_values=EmailAddress(name='Gmail Team', email='gmail-noreply@google.com'),
    to_values=(EmailAddress(name='Nicolas Fouché', email='a.b@gmail.com'),),
    cc_values=(),
    bcc_values=(),
    reply_to_values=(EmailAddress(name='', email='x.y@gmail.com'),),
)


import re
import imaplib
import datetime
from collections import UserString
from typing import Optional, List, Iterable, Sequence, Union, Tuple, Iterator

from .message import MailMessage
from .folder import MailBoxFolderManager
from .idle import IdleManager
from .consts import UID_PATTERN, PYTHON_VERSION_MINOR, MOVE_RESULT_TAG
from .utils import clean_uids, check_command_status, chunked, encode_folder, clean_flags, check_timeout_arg_support, \
    chunked_crop, StrOrBytes
from .errors import MailboxStarttlsError, MailboxLoginError, MailboxLogoutError, MailboxNumbersError, \
    MailboxFetchError, MailboxExpungeError, MailboxDeleteError, MailboxCopyError, MailboxFlagError, \
    MailboxAppendError, MailboxUidsError, MailboxTaggedResponseError, MailboxMoveError

# Maximal line length when calling readline(). This is to prevent reading arbitrary length lines.
# 20Mb is enough for search response with about 2 000 000 message numbers
imaplib._MAXLINE = 20 * 1024 * 1024  # 20Mb

Criteria = Union[StrOrBytes, UserString]


class BaseMailBox:
    """Working with the email box"""

    email_message_class = MailMessage
    folder_manager_class = MailBoxFolderManager
    idle_manager_class = IdleManager

    def __init__(self):
        self.client = self._get_mailbox_client()
        self.folder = self.folder_manager_class(self)
        self.idle = self.idle_manager_class(self)
        self.login_result = None

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, exc_traceback):
        self.logout()

    def _get_mailbox_client(self) -> imaplib.IMAP4:
        raise NotImplementedError

    def consume_until_tagged_response(self, tag: bytes):
        """Waiting for tagged response"""
        tagged_commands = self.client.tagged_commands
        response_set = []
        while True:
            response: bytes = self.client._get_response()  # noqa, example: b'IJDH3 OK IDLE Terminated'
            if tagged_commands[tag]:
                break
            response_set.append(response)
        result = tagged_commands.pop(tag)
        check_command_status(result, MailboxTaggedResponseError)
        return result, response_set

    def login(self, username: str, password: str, initial_folder: Optional[str] = 'INBOX') -> 'BaseMailBox':
        """Authenticate to account"""
        login_result = self.client._simple_command('LOGIN', username, self.client._quote(password))  # noqa
        check_command_status(login_result, MailboxLoginError)
        self.client.state = 'AUTH'  # logic from self.client.login
        if initial_folder is not None:
            self.folder.set(initial_folder)
        self.login_result = login_result
        return self  # return self in favor of context manager

    def login_utf8(self, username: str, password: str, initial_folder: Optional[str] = 'INBOX') -> 'BaseMailBox':
        """Authenticate to an account with a UTF-8 username and/or password"""
        # rfc2595 section 6 - PLAIN SASL mechanism
        encoded = (b"\0" + username.encode("utf8") + b"\0" + password.encode("utf8"))
        # Assumption is the server supports AUTH=PLAIN capability
        login_result = self.client.authenticate("PLAIN", lambda x: encoded)
        check_command_status(login_result, MailboxLoginError)
        if initial_folder is not None:
            self.folder.set(initial_folder)
        self.login_result = login_result
        return self

    def xoauth2(self, username: str, access_token: str, initial_folder: Optional[str] = 'INBOX') -> 'BaseMailBox':
        """Authenticate to account using OAuth 2.0 mechanism"""
        auth_string = f'user={username}\1auth=Bearer {access_token}\1\1'
        result = self.client.authenticate('XOAUTH2', lambda x: auth_string)  # noqa
        check_command_status(result, MailboxLoginError)
        if initial_folder is not None:
            self.folder.set(initial_folder)
        self.login_result = result
        return self

    def logout(self) -> tuple:
        """Informs the server that the client is done with the connection"""
        result = self.client.logout()
        check_command_status(result, MailboxLogoutError, expected='BYE')
        return result

    def numbers(self, criteria: Criteria = 'ALL', charset: str = 'US-ASCII') -> List[str]:
        """
        Search mailbox for matching message numbers in current folder (this is not uids)
        Message Sequence Number Message Attribute - to accessing messages by relative position in the mailbox,
        it also can be used in mathematical calculations, see rfc3501.
        :param criteria: message search criteria (see examples at ./doc/imap_search_criteria.txt)
        :param charset: IANA charset, indicates charset of the strings that appear in the search criteria. See rfc2978
        :return email message numbers
        """
        encoded_criteria = criteria if type(criteria) is bytes else str(criteria).encode(charset)
        search_result = self.client.search(charset, encoded_criteria)
        check_command_status(search_result, MailboxNumbersError)
        return search_result[1][0].decode().split() if search_result[1][0] else []

    def numbers_to_uids(self, numbers: List[str]) -> List[str]:
        """Get message uids by message numbers"""
        if not numbers:
            return []
        fetch_result = self.client.fetch(','.join(numbers), "(UID)")
        check_command_status(fetch_result, MailboxFetchError)
        result = []
        for raw_uid_item in fetch_result[1]:
            uid_match = re.search(UID_PATTERN, (raw_uid_item or b'').decode())
            if uid_match:
                result.append(uid_match.group('uid'))
        return result

    def uids(self, criteria: Criteria = 'ALL', charset: str = 'US-ASCII',
             sort: Optional[Union[str, Iterable[str]]] = None) -> List[str]:
        """
        Search mailbox for matching message uids in current folder
        :param criteria: message search criteria (see examples at ./doc/imap_search_criteria.txt)
        :param charset: IANA charset, indicates charset of the strings that appear in the search criteria. See rfc2978
        :param sort: criteria for sort messages on server, use SortCriteria constants. Charset arg is important for sort
        :return: email message uids
        """
        encoded_criteria = criteria if type(criteria) is bytes else str(criteria).encode(charset)
        if sort:
            sort = (sort,) if isinstance(sort, str) else sort
            uid_result = self.client.uid('SORT', f'({" ".join(sort)})', charset, encoded_criteria)
        else:
            uid_result = self.client.uid('SEARCH', 'CHARSET', charset, encoded_criteria)  # *charset are opt here
        check_command_status(uid_result, MailboxUidsError)
        return uid_result[1][0].decode().split() if uid_result[1][0] else []

    def _fetch_by_one(self, uid_list: Sequence[str], message_parts: str) -> Iterator[list]:
        for uid in uid_list:
            fetch_result = self.client.uid('fetch', uid, message_parts)
            check_command_status(fetch_result, MailboxFetchError)
            if not fetch_result[1] or fetch_result[1][0] is None:
                continue
            yield fetch_result[1]

    def _fetch_in_bulk(self, uid_list: Sequence[str], message_parts: str, reverse: bool, bulk: int) \
            -> Iterator[list]:
        if not uid_list:
            return

        if isinstance(bulk, int) and bulk >= 2:
            uid_list_seq = chunked_crop(uid_list, bulk)
        elif isinstance(bulk, bool):
            uid_list_seq = (uid_list,)
        else:
            raise ValueError('bulk arg may be bool or int >= 2')

        for uid_list_i in uid_list_seq:
            fetch_result = self.client.uid('fetch', ','.join(uid_list_i), message_parts)
            check_command_status(fetch_result, MailboxFetchError)
            if not fetch_result[1] or fetch_result[1][0] is None:
                return
            for built_fetch_item in chunked((reversed if reverse else iter)(fetch_result[1]), 2):
                yield built_fetch_item

    def fetch(self, criteria: Criteria = 'ALL', charset: str = 'US-ASCII', limit: Optional[Union[int, slice]] = None,
              mark_seen=True, reverse=False, headers_only=False, bulk: Union[bool, int] = False,
              sort: Optional[Union[str, Iterable[str]]] = None) \
            -> Iterator[MailMessage]:
        """
        Mail message generator in current folder by search criteria
        :param criteria: message search criteria (see examples at ./doc/imap_search_criteria.txt)
        :param charset: IANA charset, indicates charset of the strings that appear in the search criteria. See rfc2978
        :param limit: int | slice - limit number of read emails | slice emails range for read
                      useful for actions with a large number of messages, like "move" | paging
        :param mark_seen: mark emails as seen on fetch
        :param reverse: in order from the larger date to the smaller
        :param headers_only: get only email headers (without text, html, attachments)
        :param bulk:
            False - fetch each message separately per N commands - low memory consumption, slow
            True  - fetch all messages per 1 command - high memory consumption, fast. Fails on big bulk at server
            int - fetch messages by bulks of the specified size
        :param sort: criteria for sort messages on server, use SortCriteria constants. Charset arg is important for sort
        :return generator: MailMessage
        """
        message_parts = \
            f"(BODY{'' if mark_seen else '.PEEK'}[{'HEADER' if headers_only else ''}] UID FLAGS RFC822.SIZE)"
        limit_range = slice(0, limit) if type(limit) is int else limit or slice(None)
        assert type(limit_range) is slice
        uids = tuple((reversed if reverse else iter)(self.uids(criteria, charset, sort)))[limit_range]
        if bulk:
            message_generator = self._fetch_in_bulk(uids, message_parts, reverse, bulk)
        else:
            message_generator = self._fetch_by_one(uids, message_parts)
        for fetch_item in message_generator:
            yield self.email_message_class(fetch_item)

    def expunge(self) -> tuple:
        result = self.client.expunge()
        check_command_status(result, MailboxExpungeError)
        return result

    def delete(self, uid_list: Union[str, Iterable[str]], chunks: Optional[int] = None) \
            -> Optional[List[Tuple[tuple, tuple]]]:
        """
        Delete email messages
        Do nothing on empty uid_list
        :param uid_list: UIDs for delete
        :param chunks: Number of UIDs to process at once, to avoid server errors on large set. Proc all at once if None.
        :return: None on empty uid_list, command results otherwise
        """
        cleaned_uid_list = clean_uids(uid_list)
        if not cleaned_uid_list:
            return None
        results = []
        for cleaned_uid_list_i in chunked_crop(cleaned_uid_list, chunks):
            store_result = self.client.uid('STORE', ','.join(cleaned_uid_list_i), '+FLAGS', r'(\Deleted)')
            check_command_status(store_result, MailboxDeleteError)
            expunge_result = self.expunge()
            results.append((store_result, expunge_result))
        return results

    def copy(self, uid_list: Union[str, Iterable[str]], destination_folder: StrOrBytes, chunks: Optional[int] = None) \
            -> Optional[List[tuple]]:
        """
        Copy email messages into the specified folder.
        Do nothing on empty uid_list.
        :param uid_list: UIDs for copy
        :param destination_folder: Folder for email copies
        :param chunks: Number of UIDs to process at once, to avoid server errors on large set. Proc all at once if None.
        :return: None on empty uid_list, command results otherwise
        """
        cleaned_uid_list = clean_uids(uid_list)
        if not cleaned_uid_list:
            return None
        results = []
        for cleaned_uid_list_i in chunked_crop(cleaned_uid_list, chunks):
            copy_result = self.client.uid(
                'COPY', ','.join(cleaned_uid_list_i), encode_folder(destination_folder))  # noqa
            check_command_status(copy_result, MailboxCopyError)
            results.append(copy_result)
        return results

    def move(self, uid_list: Union[str, Iterable[str]], destination_folder: StrOrBytes, chunks: Optional[int] = None) \
            -> Optional[List[Tuple[tuple, tuple]]]:
        """
        Move email messages into the specified folder.
        Do nothing on empty uid_list.
        :param uid_list: UIDs for move
        :param destination_folder: Folder for move to
        :param chunks: Number of UIDs to process at once, to avoid server errors on large set. Proc all at once if None.
        :return: None on empty uid_list, command results otherwise
        """
        cleaned_uid_list = clean_uids(uid_list)
        if not cleaned_uid_list:
            return None
        if 'MOVE' in self.client.capabilities:
            # server side move
            results = []
            for cleaned_uid_list_i in chunked_crop(cleaned_uid_list, chunks):
                move_result = self.client.uid(
                    'MOVE', ','.join(cleaned_uid_list_i), encode_folder(destination_folder))  # noqa
                check_command_status(move_result, MailboxMoveError)
                results.append((move_result, MOVE_RESULT_TAG))
            return results
        else:
            # client side move
            results = []
            for cleaned_uid_list_i in chunked_crop(cleaned_uid_list, chunks):
                copy_result = self.copy(cleaned_uid_list_i, destination_folder)
                delete_result = self.delete(cleaned_uid_list_i)
                results.append((copy_result, delete_result))
            return results

    def flag(self, uid_list: Union[str, Iterable[str]], flag_set: Union[str, Iterable[str]], value: bool,
             chunks: Optional[int] = None) -> Optional[List[Tuple[tuple, tuple]]]:
        """
        Set/unset email flags.
        Do nothing on empty uid_list.
        System flags contains in consts.MailMessageFlags.all
        :param uid_list: UIDs for set flag
        :param flag_set: Flags for operate
        :param value: Should the flags be set: True - yes, False - no
        :param chunks: Number of UIDs to process at once, to avoid server errors on large set. Proc all at once if None.
        :return: None on empty uid_list, command results otherwise
        """
        cleaned_uid_list = clean_uids(uid_list)
        if not cleaned_uid_list:
            return None
        results = []
        for cleaned_uid_list_i in chunked_crop(cleaned_uid_list, chunks):
            store_result = self.client.uid(
                'STORE',
                ','.join(cleaned_uid_list_i),
                ('+' if value else '-') + 'FLAGS',
                f'({" ".join(clean_flags(flag_set))})'
            )
            check_command_status(store_result, MailboxFlagError)
            expunge_result = self.expunge()
            results.append((store_result, expunge_result))
        return results

    def append(self, message: Union[MailMessage, bytes],
               folder: StrOrBytes = 'INBOX',
               dt: Optional[datetime.datetime] = None,
               flag_set: Optional[Union[str, Iterable[str]]] = None) -> tuple:
        """
        Append email messages to server
        :param message: MailMessage object or bytes
        :param folder: destination folder, INBOX by default
        :param dt: email message datetime with tzinfo, now by default, imaplib.Time2Internaldate types supported
        :param flag_set: email message flags, no flags by default. System flags at consts.MailMessageFlags.all
        :return: command results
        """
        if PYTHON_VERSION_MINOR < 6:
            timezone = datetime.timezone(datetime.timedelta(hours=0))
        else:
            timezone = datetime.datetime.now().astimezone().tzinfo  # system timezone
        cleaned_flags = clean_flags(flag_set or [])
        typ, dat = self.client.append(
            encode_folder(folder),  # noqa
            f'({" ".join(cleaned_flags)})' if cleaned_flags else None,
            dt or datetime.datetime.now(timezone),  # noqa
            message if type(message) is bytes else message.obj.as_bytes()
        )
        append_result = (typ, dat)
        check_command_status(append_result, MailboxAppendError)
        return append_result


class MailBoxUnencrypted(BaseMailBox):
    """Working with the email box through IMAP4"""

    def __init__(self, host='', port=143, timeout=None):
        """
        :param host: host's name (default: localhost)
        :param port: port number
        :param timeout: timeout in seconds for the connection attempt, since python 3.9
        """
        check_timeout_arg_support(timeout)
        self._host = host
        self._port = port
        self._timeout = timeout
        super().__init__()

    def _get_mailbox_client(self) -> imaplib.IMAP4:
        if PYTHON_VERSION_MINOR < 9:
            return imaplib.IMAP4(self._host, self._port)
        elif PYTHON_VERSION_MINOR < 12:
            return imaplib.IMAP4(self._host, self._port, self._timeout)
        else:
            return imaplib.IMAP4(self._host, self._port, timeout=self._timeout)


class MailBox(BaseMailBox):
    """Working with the email box through IMAP4 over SSL connection"""

    def __init__(self, host='', port=993, timeout=None, keyfile=None, certfile=None, ssl_context=None):
        """
        :param host: host's name (default: localhost)
        :param port: port number
        :param timeout: timeout in seconds for the connection attempt, since python 3.9
        :param keyfile: PEM formatted file that contains your private key (deprecated)
        :param certfile: PEM formatted certificate chain file (deprecated)
        :param ssl_context: SSLContext object that contains your certificate chain and private key
        Since Python 3.9 timeout argument added
        Since Python 3.12 keyfile and certfile arguments are deprecated, ssl_context and timeout must be keyword args
        """
        check_timeout_arg_support(timeout)
        self._host = host
        self._port = port
        self._timeout = timeout
        self._keyfile = keyfile
        self._certfile = certfile
        self._ssl_context = ssl_context
        super().__init__()

    def _get_mailbox_client(self) -> imaplib.IMAP4:
        if PYTHON_VERSION_MINOR < 9:
            return imaplib.IMAP4_SSL(self._host, self._port, self._keyfile, self._certfile, self._ssl_context)  # noqa
        elif PYTHON_VERSION_MINOR < 12:
            return imaplib.IMAP4_SSL(
                self._host, self._port, self._keyfile, self._certfile, self._ssl_context, self._timeout)  # noqa
        else:
            return imaplib.IMAP4_SSL(self._host, self._port, ssl_context=self._ssl_context, timeout=self._timeout)


class MailBoxTls(BaseMailBox):
    """Working with the email box through IMAP4 with STARTTLS"""

    def __init__(self, host='', port=993, timeout=None, ssl_context=None):
        """
        :param host: host's name (default: localhost)
        :param port: port number
        :param timeout: timeout in seconds for the connection attempt, since python 3.9
        :param ssl_context: SSLContext object that contains your certificate chain and private key
        """
        check_timeout_arg_support(timeout)
        self._host = host
        self._port = port
        self._timeout = timeout
        self._ssl_context = ssl_context
        super().__init__()

    def _get_mailbox_client(self) -> imaplib.IMAP4:
        if PYTHON_VERSION_MINOR < 9:
            client = imaplib.IMAP4(self._host, self._port)
        elif PYTHON_VERSION_MINOR < 12:
            client = imaplib.IMAP4(self._host, self._port, self._timeout)
        else:
            client = imaplib.IMAP4(self._host, self._port, timeout=self._timeout)
        result = client.starttls(self._ssl_context)
        check_command_status(result, MailboxStarttlsError)
        return client
